@@ -27,7 +27,7 @@
 #     Anton Berezin  <tobez@tobez.org>
 #     Dmitry Karasik <dk@plab.ku.dk> 
 #
-#  $Id: Classes.pm,v 1.104 2008/05/06 09:34:03 dk Exp $
+#  $Id: Classes.pm,v 1.106 2009/09/22 13:48:25 dk Exp $
 use Prima;
 use Prima::Const;
 
@@ -230,7 +230,8 @@ sub AUTOLOAD
 	no strict;
 	my $self = shift;
 	my $expectedMethod = $AUTOLOAD;
-	Carp::confess "There is no such thing as \"$expectedMethod\"\n" if scalar @_;
+	Carp::confess "There is no such thing as \"$expectedMethod\"\n"
+		if scalar(@_) or not ref $self;
 	my ($componentName) = $expectedMethod =~ /::([^:]+)$/;
 	my $component = $self-> bring( $componentName);
 	Carp::croak("Unknown widget or method \"$expectedMethod\"") 
@@ -238,6 +239,17 @@ sub AUTOLOAD
 	return $component;
 }
 
+sub find_component
+{
+	my ( $self, $name ) = @_;
+	my @q = $self-> get_components;
+	while ( my $x = shift @q ) {
+		return $x if $x-> name eq $name;
+		push @q, $x-> get_components;
+	}
+	return undef;
+}
+
 # class File
 package Prima::File;
 use vars qw(@ISA);
@@ -1412,7 +1424,8 @@ sub AUTOLOAD
 	no strict;
 	my $self = shift;
 	my $expectedMethod = $AUTOLOAD;
-	die "There is no such method as \"$expectedMethod\"" if scalar @_;
+	die "There is no such method as \"$expectedMethod\""
+		if scalar(@_) or not ref $self;
 	my ($itemName) = $expectedMethod =~ /::([^:]+)$/;
 	die "Unknown menu item identifier \"$itemName\"" 
 		unless defined $itemName && $self-> has_item( $itemName);
