@@ -1,3 +1,28 @@
+#  Copyright (c) 2008 Dmitry Karasik
+#  All rights reserved.
+#
+#  Redistribution and use in source and binary forms, with or without
+#  modification, are permitted provided that the following conditions
+#  are met:
+#  1. Redistributions of source code must retain the above copyright
+#     notice, this list of conditions and the following disclaimer.
+#  2. Redistributions in binary form must reproduce the above copyright
+#     notice, this list of conditions and the following disclaimer in the
+#     documentation and/or other materials provided with the distribution.
+#
+#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+#  SUCH DAMAGE.
+#
+# $Id: AnimateGIF.pm,v 1.3 2010/02/09 18:10:22 dk Exp $
 package Prima::Image::AnimateGIF;
 
 use strict;
@@ -142,7 +167,7 @@ sub reset
 	$self-> {canvas}  = Prima::DeviceBitmap-> new(
 		width      => $e-> {screenWidth},
 		height     => $e-> {screenHeight},
-		type       => dbt::Pixmap,
+		monochrome => 0,
 		backColor  => 0,
 	);
 	$self-> {canvas}-> clear; # canvas is all-0 initially
@@ -150,7 +175,7 @@ sub reset
 	$self-> {mask}    = Prima::DeviceBitmap-> new(
 		width      => $e-> {screenWidth},
 		height     => $e-> {screenHeight},
-		type       => dbt::Bitmap,
+		monochrome => 1,
 		backColor  => 0xFFFFFF,
 		color      => 0x000000,
 	);
@@ -234,7 +259,7 @@ sub next
 		my $c  = Prima::DeviceBitmap-> new(
 			width      => $sz[0],
 			height     => $sz[1],
-			type       => dbt::Pixmap,
+			monochrome => 0,
 		);
 		$c-> put_image( 0, 0, $self-> {canvas});
 		$self-> {saveCanvas} = $self-> {canvas};
@@ -243,7 +268,7 @@ sub next
 		$c = Prima::DeviceBitmap-> new(
 			width      => $sz[0],
 			height     => $sz[1],
-			type       => dbt::Bitmap,
+			monochrome => 1,
 		);
 		$c-> put_image( 0, 0, $self-> {mask});
 		$self-> {saveMask} = $self-> {mask};
@@ -507,7 +532,7 @@ Returns width of the composite frame.
 =head1 SEE ALSO
 
 L<Prima::image-load>,
-L<http://www.the-labs.com/GIFMerge/>
+L<http://www.the-labs.com/GIFMerge/ >
 
 =head1 AUTHOR
 
