@@ -1,12 +1,6 @@
-=head1 NAME
-
-examples/animate.pl - Animate using L<Prima::Image::Animate>
-
-=cut
-
+# $Id: animate.pl,v 1.2 2008/04/25 08:17:35 dk Exp $
 use strict;
-use warnings;
-use Prima qw(Application Image::Animate);
+use Prima qw(Application Image::AnimateGIF);
 
 my $clip_debug = 0;
 
@@ -25,7 +19,7 @@ unless ( @ARGV) {
 	}
 }
 
-my $x = Prima::Image::Animate->load($ARGV[0]);
+my $x = Prima::Image::AnimateGIF->load($ARGV[0]);
 die "Can't load $ARGV[0]:$@\n" unless $x;
 
 my ( $X, $Y) = ( 100, 100);
@@ -34,13 +28,11 @@ my $g = $::application-> get_image( $X, $Y, $x-> size);
 $::application-> begin_paint;
 
 my $break;
-my $with_background = ($ARGV[1] // '') eq '-bg';
 
 $SIG{INT} = sub { $break++ };
 while ( my $info = $x-> next) {
 	my $c = $g-> dup;
 	$c-> begin_paint;
-	$x-> draw_background( $c, 0, 0) if $with_background;
 	$x-> draw( $c, 0, 0);
 
 	$::application-> clipRect(
@@ -65,7 +57,7 @@ while ( my $info = $x-> next) {
 	}
 	$::application-> rop(rop::CopyPut);
 	$::application-> put_image( $X, $Y, $c);
-
+	
 	$::application-> sync;
 	select(undef, undef, undef, $info-> {delay});
 
