@@ -24,14 +24,16 @@
  * SUCH DAMAGE.
  */
 /* Created by Dmitry Karasik <dk@plab.ku.dk> */
-/* $Id: codec_png.c,v 1.9 2004/11/25 15:11:56 dk Exp $ */
+/* $Id: codec_png.c,v 1.19 2010/06/15 17:30:39 dk Exp $ */
 
+#define USE_NO_MINGW_SETJMP_TWO_ARGS
 #ifdef HAVE_CONFIG_H
 #include <generic/config.h>
 #endif
 #define Z_PREFIX 
 #include <png.h>
 #undef Byte
+#undef FAR
 
 #ifndef PNG_GAMMA_THRESHOLD
 #define PNG_GAMMA_THRESHOLD 0.05
@@ -44,10 +46,17 @@
 #  define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
 #endif
 
+
 #include "img.h"
 #include "img_conv.h"
 #include "Icon.h"
 
+#ifdef BROKEN_PERL_PLATFORM
+#undef      setjmp
+#undef      longjmp
+#define     setjmp _setjmp
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -154,10 +163,7 @@ static ImgCodecInfo codec_info = {
    features,    /* features  */
    "",     /* module */
    "",     /* package */
-   true,   /* canLoad */
-   false,  /* canLoadMultiple  */
-   true,   /* canSave */
-   false,  /* canSaveMultiple */
+   IMG_LOAD_FROM_FILE | IMG_LOAD_FROM_STREAM | IMG_SAVE_TO_FILE | IMG_SAVE_TO_STREAM,
    pngbpp, /* save types */
    loadOutput
 };
@@ -257,21 +263,19 @@ typedef struct _LoadRec {
    Byte * line;
 } LoadRec;
 
-static 
+static void
 #ifdef PNGAPI 
 PNGAPI
 #endif
-void
 warning_fn( png_structp png_ptr, png_const_charp msg) 
 { 
    /* warn( msg); */
 }
 
-static 
+static void
 #ifdef PNGAPI 
 PNGAPI
 #endif
-void
 error_fn( png_structp png_ptr, png_const_charp msg) 
 {
    char * buf = ( char *) png_get_error_ptr( png_ptr); 
@@ -279,15 +283,39 @@ error_fn( png_structp png_ptr, png_const_charp msg)
    longjmp( png_ptr-> jmpbuf, 1);
 }
 
+static void
+img_png_read (png_structp png_ptr, png_bytep data, png_size_t size)
+{
+   req_read( (( PImgLoadFileInstance) png_ptr-> io_ptr)-> req, size, data);
+}
+
+static void
+img_png_write (png_structp png_ptr, png_bytep data, png_size_t size)
+{
+   req_write( (( PImgLoadFileInstance) png_ptr-> io_ptr)-> req, size, data);
+}
+
+static void
+img_png_flush (png_structp png_ptr)
+{
+   req_flush( (( PImgLoadFileInstance) png_ptr-> io_ptr)-> req);
+}
+
 static void * 
 open_load( PImgCodec instance, PImgLoadFileInstance fi)
 {
    LoadRec * l;
    unsigned char buf[8];
 
-   if ( fseek( fi-> f, 0, SEEK_SET) < 0) return false;
-   if ( fread( buf, 1, 8, fi-> f) != 8) return false;
-   if ( png_sig_cmp( buf, 0, 8) != 0) return false;
+   if ( req_seek( fi-> req, 0, SEEK_SET) < 0) return false;
+   if ( req_read( fi-> req, 8, buf) < 0) {
+      req_seek( fi-> req, 0, SEEK_SET);
+      return false;
+   }
+   if ( png_sig_cmp( buf, 0, 8) != 0) {
+      req_seek( fi-> req, 0, SEEK_SET);
+      return false;
+   }
 
    fi-> stop = true;
    fi-> frameCount = 1;
@@ -318,7 +346,7 @@ open_load( PImgCodec instance, PImgLoadFileInstance fi)
       return false;
    }
 
-   png_init_io( l-> png_ptr, fi-> f);
+   png_set_read_fn( l-> png_ptr, fi, img_png_read);
    png_set_sig_bytes( l-> png_ptr, 8);
    return l;
 }
@@ -326,6 +354,7 @@ open_load( PImgCodec instance, PImgLoadFileInstance fi)
 static Bool   
 load( PImgCodec instance, PImgLoadFileInstance fi)
 {
+   dPROFILE;
    LoadRec * l = ( LoadRec *) fi-> instance;
    png_uint_32 width, height;
    int obd, bit_depth, color_type, interlace_type, bpp, number_passes, pass, filter;
@@ -544,8 +573,8 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
    } 
 
    if ( fi-> noImageData) {
-      hv_store( fi-> frameProperties, "width",  5, newSViv( width), 0);
-      hv_store( fi-> frameProperties, "height", 6, newSViv( height), 0);
+      (void) hv_store( fi-> frameProperties, "width",  5, newSViv( width), 0);
+      (void) hv_store( fi-> frameProperties, "height", 6, newSViv( height), 0);
       if ( fi-> loadExtras) { /* skip data and read info blocks */
          for (pass = 0; pass < number_passes; pass++) {
             int y;
@@ -560,6 +589,8 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
    /* reading bits */
    CImage( fi-> object)-> create_empty( fi-> object, width, height, bpp);
 
+   EVENT_HEADER_READY(fi);
+
    /* create buffer for 8 to 4 bpp conversion */
    if ( obd == 2) 
       if ( !( l-> b8_4 = malloc( width))) outcm( width);
@@ -600,12 +631,13 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
       Byte * data = PImage( fi-> object)-> data;
       Byte * a_data = nil;
       
+      EVENT_SCANLINES_RESET(fi);
       data += ( height - 1) * PImage( fi-> object)-> lineSize;
       if ( alpha_image) {
          a_data = PImage( alpha_image)-> data;
          a_data += ( height - 1) * PImage( alpha_image)-> lineSize;
       }
-      for (y = 0; y < height; y++) { 
+      for (y = 0; y < height; y++) {
          if ( alpha_image) {
             int i;
             Byte * dst = data, * src = l-> line, *a = a_data;
@@ -632,7 +664,10 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
          }
          data -= PImage( fi-> object)-> lineSize;
          if ( alpha_image) a_data -= PImage( alpha_image)-> lineSize;
+
+         EVENT_TOPDOWN_SCANLINES_READY(fi,1);
       }
+      EVENT_SCANLINES_FINISHED(fi);
    }
 
    /* adjusting icon mask if possible */
@@ -656,10 +691,10 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
                    min_ix = i;
                 }
              p += min_ix;
+             PIcon( fi-> object)-> maskIndex = min_ix;
           } else 
-             p += trns_p[0].index;
-          PIcon( fi-> object)-> maskColor   = ARGB( p->r, p->g, p-> b);
-          PIcon( fi-> object)-> autoMasking = amMaskColor;
+             PIcon( fi-> object)-> maskIndex = trns_p[0].index;
+          PIcon( fi-> object)-> autoMasking = amMaskIndex;
       }
    }
 
@@ -712,7 +747,7 @@ READ_END:
       if ( png_get_text( l-> png_ptr, l-> info_ptr, &tx, &ct)) {
          HV * hash = newHV();
          for ( i = 0; i < ct; i++, tx++) 
-            hv_store( hash, tx-> key, strlen( tx-> key), newSVpv( tx-> text, tx-> text_length), 0);
+            (void) hv_store( hash, tx-> key, strlen( tx-> key), newSVpv( tx-> text, tx-> text_length), 0);
          pset_sv_noinc( text, newRV_noinc(( SV *) hash));
       }
 #endif      
@@ -840,8 +875,7 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
       free( l);
       return false;
    }
-
-   png_init_io( l-> png_ptr, fi-> f);
+   png_set_write_fn( l-> png_ptr, fi, img_png_write, img_png_flush);
 
    return l;
 }
@@ -849,6 +883,7 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
 static Bool   
 save( PImgCodec instance, PImgSaveFileInstance fi)
 {
+   dPROFILE;
    PIcon i = ( PIcon) fi-> object;
    SaveRec * l = ( SaveRec *) fi-> instance;
    HV * profile = fi-> objectExtras;
@@ -990,25 +1025,37 @@ save( PImgCodec instance, PImgSaveFileInstance fi)
       png_byte trns_t[256];
       int trns_n = 0;
       Color color = pexist( transparent_color) ? pget_i( transparent_color) : clInvalid;
+      int   index = pexist( transparent_color_index) ? pget_i( transparent_color_index) : -1;
 
-      if ( icon && (i-> autoMasking == amMaskColor) && ( color & clSysFlag))
+      if ( index > i-> palSize)
+         index = i-> palSize - 1;
+      if ( icon && (i-> autoMasking == amMaskIndex) && ( index < 0))
+         index = i-> maskIndex;
+
+      if ( color & clSysFlag)
+         color = clInvalid;
+      if ( icon && (i-> autoMasking == amMaskColor) && ( color == clInvalid))
          color = i-> maskColor;
       memset( trns_p, 0, sizeof( trns_p));
-      if ( !( color & clSysFlag)) {
+      if ( color != clInvalid || index >= 0) {
          memset( trns_t, 255, sizeof( trns_t));
          if (( i-> type & imBPP) < 24) {
             int x;
-            RGBColor r;
-            r.r = (color & 0xFF0000) >> 16;
-            r.g = (color & 0x00FF00) >> 8;
-            r.b = (color & 0x0000FF);
-            x = cm_nearest_color( r, i-> palSize, i-> palette);
+	    if ( index < 0) {
+               RGBColor r;
+               r.r = (color & 0xFF0000) >> 16;
+               r.g = (color & 0x00FF00) >> 8;
+               r.b = (color & 0x0000FF);
+               x = cm_nearest_color( r, i-> palSize, i-> palette);
+	    } else {
+	       x = index;
+	    }
             trns_t[x] = 0;
             trns_n = x + 1;
             trns_p[0].index = x;
             trns_p[x].index = x;
             png_set_tRNS(l->png_ptr, l-> info_ptr, trns_t, trns_n, trns_p);
-         } else {
+         } else if ( color != clInvalid) {
             trns_p[0].red   = (color & 0xFF0000) >> 16;
             trns_p[0].green = (color & 0x00FF00) >> 8;
             trns_p[0].blue  = (color & 0x0000FF);
