@@ -26,12 +26,15 @@
  * Created by Dmitry Karasik <dmitry@karasik.eu.org> with great help
  * of tiff2png.c by Willem van Schaik and Greg Roelofs
  *
- * $Id: codec_tiff.c,v 1.4 2004/11/25 15:11:56 dk Exp $
+ * $Id: codec_tiff.c,v 1.15 2011/01/09 23:24:41 dk Exp $
  */
 
 #include "img.h"
 #include "img_conv.h"
 #include "Icon.h"
+#if defined(_MSC_VER) && _MSC_VER < 1500
+#define HAVE_INT32
+#endif
 #include <tiff.h>
 #include <tiffio.h>
 #include <tiffconf.h>
@@ -168,10 +171,8 @@ static ImgCodecInfo codec_info = {
    tifffeatures,    /* features  */
    "Prima::Image::tiff",     /* module */
    "Prima::Image::tiff",     /* package */
-   true,   /* canLoad */
-   true,  /* canLoadMultiple  */
-   true,   /* canSave */
-   true,  /* canSaveMultiple */
+   IMG_LOAD_FROM_FILE | IMG_LOAD_MULTIFRAME | IMG_LOAD_FROM_STREAM | 
+   IMG_SAVE_TO_FILE | IMG_SAVE_MULTIFRAME | IMG_SAVE_TO_STREAM,
    tiffbpp, /* save types */
    loadOutput
 };
@@ -198,14 +199,63 @@ load_defaults( PImgCodec c)
    return profile;
 }
 
+static tsize_t
+my_tiff_read( thandle_t h, tdata_t data, tsize_t size)
+{
+    return req_read( (PImgIORequest) h, size, data);
+}
+
+static tsize_t
+my_tiff_write( thandle_t h, tdata_t data, tsize_t size)
+{
+    return req_write( (PImgIORequest) h, size, data);
+}
+
+static toff_t
+my_tiff_seek( thandle_t h, toff_t offset, int whence)
+{
+    if ( req_seek( (PImgIORequest) h, offset, whence) < 0)
+       return -1;
+    return req_tell( (PImgIORequest) h);
+}
+
+static int
+my_tiff_close( thandle_t h)
+{
+    return (( PImgIORequest) h)-> flush ? 
+       req_flush( (PImgIORequest) h) :
+       0;
+}
+
+static toff_t
+my_tiff_size( thandle_t h)
+{
+    return 0;
+}
+
+static int
+my_tiff_map( thandle_t h, tdata_t * data, toff_t * offset)
+{
+    return 0;
+}
+
+static void
+my_tiff_unmap( thandle_t h, tdata_t data, toff_t offset)
+{
+}
+
+
 static void * 
 open_load( PImgCodec instance, PImgLoadFileInstance fi)
 {
    TIFF * tiff;
    errbuf = fi-> errbuf;
    err_signal = 0;
-   if (!( tiff = TIFFFdOpen( fileno(fi-> f), fi-> fileName, "r"))) {
-      fseek( fi-> f, 0, SEEK_SET);
+   if (!( tiff = TIFFClientOpen( "", "r", (thandle_t) fi-> req,
+      my_tiff_read, my_tiff_write,
+      my_tiff_seek, my_tiff_close, my_tiff_size, 
+      my_tiff_map, my_tiff_unmap))) {
+      req_seek( fi-> req, 0, SEEK_SET);
       return nil;
    }
    fi-> frameCount = TIFFNumberOfDirectories( tiff);
@@ -266,19 +316,19 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
    uint16 resunit;
    char * photometric_descr = nil;
    unsigned short photometric, bps, spp, planar, comp_method;
-   int x, y, w, h, bpp = 0, palSize = 0, icon, tiled, 
+   int x, y, w, h, bpp = 0, palSize = 0, icon, tiled, rgba_striped = 0,
       InvertMinIsWhite = INVERT_MINISWHITE, strip_bps, faxpect = 0;
    float xres, yres;
    unsigned short *redcolormap, *greencolormap, *bluecolormap;
    Byte *tiffstrip, *tiffline, *tifftile, *primaline, *primamask = nil;
    size_t stripsz, linesz, tilesz = 0L;
-   uint32 tile_width, tile_height, num_tilesX = 0L;
+   uint32 tile_width, tile_height, num_tilesX = 0L, rowsperstrip;
    Byte bw_colorref[256];
    
    errbuf = fi-> errbuf;
    err_signal = 0;
    
-   if ( !TIFFSetDirectory( tiff, fi-> frame)) {
+   if ( !TIFFSetDirectory( tiff, (tdir_t) fi-> frame)) {
       outc( "Frame index out of range");
       return false;
    }
@@ -295,6 +345,7 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
       outc("Cannot query IMAGELENGTH tag");
       return false;
    }
+
    if ( !TIFFGetField( tiff, TIFFTAG_BITSPERSAMPLE, &bps))  bps = 1;
       else if ( fi-> loadExtras) pset_i( BitsPerSample, bps);
    if ( bps != 16 && bps != 8 && bps != 4 && bps != 2 && bps != 1) {
@@ -409,29 +460,27 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
       break;
 #ifdef JPEG_SUPPORT
    case PHOTOMETRIC_SEPARATED:
-      /* probably can rely on libjpeg to convert to RGB */
       bpp = imbpp24;
-      spp = 3;
-      TIFFSetField( tiff, TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB);
+      spp = 4;
+      rgba_striped = 1;
       photometric_descr = "Separated";
       photometric = PHOTOMETRIC_RGB;
       break;
 #endif
-   case PHOTOMETRIC_MASK:
-   case PHOTOMETRIC_CIELAB:
-   case PHOTOMETRIC_DEPTH:
-      sprintf( fi-> errbuf, 
-        "Don't know how to handle photometric %s",
+   default:
+      /* fallback, to RGBA strips */
+      bpp = imbpp24;
+      spp = 4;
+      rgba_striped = 1;
+      photometric_descr = 
         photometric == PHOTOMETRIC_MASK?      "MASK" :
         photometric == PHOTOMETRIC_CIELAB?    "CIELAB" :
         photometric == PHOTOMETRIC_DEPTH?     "DEPTH" :
         photometric == PHOTOMETRIC_SEPARATED? "SEPARATED" :
         photometric == PHOTOMETRIC_YCBCR?     "YCBCR" :
-                                              "unknown");
-       return false;
-   default:
-      sprintf( fi-> errbuf, "Unknown PHOTOMETRIC=%d", photometric);
-      return false;
+                                              "unknown";
+      photometric = PHOTOMETRIC_RGB;
+      break;
    }
 
    /* check bps and spp combinations - 3 and 4 samples for RGB, 1 and 2 for the others */
@@ -494,6 +543,7 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
    
    /* check load options */   
    {
+      dPROFILE;
       HV * profile = fi-> profile;
       if ( pexist( InvertMinIsWhite)) InvertMinIsWhite = pget_i( InvertMinIsWhite);
   /* check fax option applicability */
@@ -519,7 +569,7 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
       pset_i( height, h);
    } else 
       CImage( fi-> object)-> create_empty( fi-> object, w, h, bpp);
-      
+   EVENT_HEADER_READY(fi);
 
    /* check if palette available */
    if ( photometric == PHOTOMETRIC_PALETTE) { 
@@ -599,21 +649,30 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
       /* check if linesz is big enough */
       z = tilesz / tile_height * num_tilesX;
       if ( linesz < z) linesz = z;
+
+      rowsperstrip = 1;
    } else {
       tile_width  = w;
       tile_height = 1;
       num_tilesX  = 1;
       tilesz      = linesz;
+      if ( rgba_striped) {
+         if( !TIFFGetField(tiff, TIFFTAG_ROWSPERSTRIP, &rowsperstrip) ) {
+             outc("Cannot query ROWSPERSTRIP tag");
+             return false;
+         }
+      } else
+	 rowsperstrip = 1;
    }
 
 
    /* setup buffers for twofold size for byte and intrapixel conversion */
    strip_bps = ( bps > 8) ? 2 : 1;
-   if ( !( tifftile = (Byte*) malloc( strip_bps * w * spp * 2))) {
+   if ( !( tifftile = (Byte*) malloc( strip_bps * w * rowsperstrip * tile_height * spp * 2))) {
       outcm( strip_bps * w * spp * 2);
       return false;
    }
-   stripsz = strip_bps * tile_height * w * spp;
+   stripsz = strip_bps * rowsperstrip * tile_height * w * spp;
    if ( !( tiffstrip = (Byte*) malloc( stripsz * 2))) {
       free( tifftile);
       outcm( stripsz * 2);
@@ -640,31 +699,64 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
          if (( y % tile_height) == 0) {
             for (col = 0; ok && col < num_tilesX; col++) {
                Byte *dest, *src;
-               int r, dd, sd;
+               int r, dd, sd, rows, cols;
                int tileno = col+(y/tile_height)*num_tilesX;
                /* read the tile into the array */
-               if (!TIFFReadEncodedTile(tiff, tileno, tifftile, tilesz)) {
+	       int ret = rgba_striped ?
+		  TIFFReadRGBATile( tiff, col * tile_width, y, (void*) tifftile) :
+                  TIFFReadEncodedTile(tiff, tileno, tifftile, tilesz);
+               if (!ret) {
                   ok = 0;
                   break;
                }
 
                /* copy this tile into the row buffer */
-               src  = tifftile;
-               dest = tiffstrip + stripsz + col * strip_bps * spp;
-               dd   = linesz;
-               sd   = tilesz / tile_height;
-               for (r = 0; r < (int) tile_height; r++, src += sd, dest += dd) 
-                  scan_convert( src, dest, tile_width * spp, bps);
+               dest = tiffstrip + stripsz + col * strip_bps * spp * tile_width;
+	       rows = ((y + tile_height) > h) ? h - y : tile_height;
+	       cols = (col == num_tilesX - 1) ? w - col * tile_width : tile_width;
+               dd   = w * spp;
+	       if ( rgba_striped) {
+	          /* RGBATiles are reversed */
+		  sd   = - (tile_width * spp);
+                  src  = tifftile - sd * (tile_height - 1); 
+	       } else {
+                  sd   = tilesz / tile_height;
+                  src  = tifftile;
+	       }
+               for (r = 0; r < rows; r++, src += sd, dest += dd)
+                  scan_convert( src, dest, cols * spp, bps);
             }
             tiffline = tiffstrip; /* set tileline to top of strip */
          } else 
-            tiffline = tiffstrip + (y % tile_height) * linesz;
+            tiffline = tiffstrip + (y % tile_height) * w * spp;
+      } else if ( rgba_striped) {
+         /* Is it time for a new strip? */
+         if (( y % rowsperstrip) == 0) {
+            Byte *dest, *src;
+            int r, rows, dd, sd;
+            if ( TIFFReadRGBAStrip( tiff, y, (void*) tifftile) < 0) {
+               if ( !( errbuf && errbuf[0]))
+                 sprintf( fi-> errbuf, "Error reading scanline %d", y);
+               free(tifftile);
+               free(tiffstrip);
+               return false;
+	    }
+	    rows = ((y + rowsperstrip) > h) ? h - y : rowsperstrip;
+            dest = tiffstrip + stripsz;
+            dd   = sd = spp * w;
+            src  = tifftile + sd * (rows - 1);
+	    /* RGBAStrips are reversed */
+            for (r = 0; r < rows; r++, src -= sd, dest += dd) 
+                scan_convert( src, dest, sd, bps);
+            tiffline = tiffstrip; /* set tileline to top of strip */
+	 } else
+            tiffline = tiffstrip + (y % rowsperstrip) * spp * w;
       } else {
          int s = 0, reads = ( planar == PLANARCONFIG_CONTIG) ? 1 : spp;
          int dw = w * (( planar == PLANARCONFIG_CONTIG) ? spp : 1);
          Byte * d = tiffline + stripsz;
          for ( s = 0; s < reads; s++, d += w * strip_bps) {
-            if ( TIFFReadScanline( tiff, tiffline, y, s) < 0) {
+            if ( TIFFReadScanline( tiff, tiffline, y, (tsample_t) s) < 0) {
                if ( !( errbuf && errbuf[0]))
                  sprintf( fi-> errbuf, "Error reading scanline %d", y);
                free(tifftile);
@@ -787,8 +879,9 @@ load( PImgCodec instance, PImgLoadFileInstance fi)
          bc_byte_mono_cr( alpha, primamask, w, bw_colorref);
          primamask -= i-> maskLine;
       }
+      EVENT_TOPDOWN_SCANLINES_READY(fi,1);
    }
-
+   EVENT_SCANLINES_FINISHED(fi);
    
    /* finalize */
    free( tiffstrip);
@@ -838,7 +931,10 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
    TIFF * tiff;
    errbuf = fi-> errbuf;
    err_signal = 0;
-   if (!( tiff = TIFFFdOpen( fileno(fi-> f), fi-> fileName, "w")))
+   if (!( tiff = TIFFClientOpen( "", "w", (thandle_t) fi-> req,
+      my_tiff_read, my_tiff_write,
+      my_tiff_seek, my_tiff_close, my_tiff_size, 
+      my_tiff_map, my_tiff_unmap)))
       return nil;
    return tiff;
 }
@@ -846,6 +942,7 @@ open_save( PImgCodec instance, PImgSaveFileInstance fi)
 static Bool   
 save( PImgCodec instance, PImgSaveFileInstance fi)
 {
+   dPROFILE;
    PIcon i = ( PIcon) fi-> object;
    TIFF * tiff = ( TIFF*) fi-> instance;
    Bool icon = kind_of( fi-> object, CIcon);
