@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: img.h,v 1.13 2008/04/26 11:19:58 dk Exp $
+ * $Id: img.h,v 1.15 2009/10/23 17:22:28 dk Exp $
  */
 /* Created by Dmitry Karasik <dk@plab.ku.dk> */
 
@@ -71,12 +71,14 @@ typedef struct _ImgLoadFileInstance {
   int             frameCount;     /* total frames in the file; can return -1 if unknown */
   HV            * fileProperties; /* specific file data */
   void          * instance;       /* user instance */
+  Bool            wasTruncated;   /* if codec can recover from EOF */
 
   /* user-specified data - applied to whole file */
   Bool            loadExtras; 
   Bool            loadAll;
   Bool            noImageData;
   Bool            iconUnmask;
+  Bool            noIncomplete;
   HV            * extras;         /* profile applied to all frames */
 
   /* user-specified data - applied to every frame */
@@ -130,6 +132,7 @@ typedef struct _ImgSaveFileInstance {
 #define IMG_SAVE_TO_FILE             0x0000010
 #define IMG_SAVE_TO_STREAM           0x0000020
 #define IMG_SAVE_MULTIFRAME          0x0000040
+#define IMG_SAVE_APPEND              0x0000080
 
 /* codec info */
 typedef struct _ImgCodecInfo {
