@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: apc_graphics.c,v 1.120 2006/11/02 13:39:42 dk Exp $
+ * $Id: apc_graphics.c,v 1.124 2010/12/07 12:54:21 dk Exp $
  */
 
 /***********************************************************/
@@ -586,6 +586,7 @@ apc_gp_arc( Handle self, int x, int y, int dX, int dY, double angleStart, double
    if ( needf)
       XDrawArc( DISP, XX-> gdrawable, XX-> gc, ELLIPSE_RECT,
           angleStart * 64, ( angleEnd - angleStart) * 64);
+   XFLUSH;	  
    return true;
 }
 
@@ -604,6 +605,7 @@ apc_gp_bar( Handle self, int x1, int y1, int x2, int y2)
    while ( prima_make_brush( XX, mix++)) 
       XFillRectangle( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y2), x2 - x1 + 1, y2 - y1 + 1);
    XCHECKPOINT;
+   XFLUSH;	  
    return true;
 }
 
@@ -644,6 +646,7 @@ apc_gp_clear( Handle self, int x1, int y1, int x2, int y2)
       XSetFillStyle( DISP, XX-> gc, FillSolid);
    XX-> flags. brush_fore = 0;
    XFillRectangle( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y2), x2 - x1 + 1, y2 - y1 + 1);
+   XFLUSH;	  
    
    return true;
 }
@@ -674,6 +677,7 @@ apc_gp_chord( Handle self, int x, int y, int dX, int dY, double angleStart, doub
    XDrawLine( DISP, XX-> gdrawable,
       XX-> gc, x + cos( angleStart / GRAD) * dX / 2, y - sin( angleStart / GRAD) * dY / 2,
                x + cos( angleEnd / GRAD) * dX / 2,   y - sin( angleEnd / GRAD) * dY / 2);
+   XFLUSH;	  
    return true;
 }
 
@@ -702,6 +706,7 @@ apc_gp_draw_poly( Handle self, int n, Point *pp)
    XDrawLines( DISP, XX-> gdrawable, XX-> gc, p, n, CoordModeOrigin);
 
    free( p);
+   XFLUSH;	  
    return true;
 }
 
@@ -732,6 +737,7 @@ apc_gp_draw_poly2( Handle self, int np, Point *pp)
    XDrawSegments( DISP, XX-> gdrawable, XX-> gc, s, n);
 
    free( s);
+   XFLUSH;	  
    return true;
 }
 
@@ -740,6 +746,9 @@ apc_gp_ellipse( Handle self, int x, int y, int dX, int dY)
 {
    DEFXX;
 
+   if ( dX == 1 || dY == 1 ) /* Xorg bug */
+   	return apc_gp_rectangle( self, x - dX / 2, y - dY / 2, x + dX / 2, y + dY / 2);
+
    if ( PObject( self)-> options. optInDrawInfo) return false;
    if ( !XF_IN_PAINT(XX)) return false;
    if ( dX <= 0 || dY <= 0) return false;
@@ -750,6 +759,7 @@ apc_gp_ellipse( Handle self, int x, int y, int dX, int dY)
    PURE_FOREGROUND;
    calculate_ellipse_divergence();
    XDrawArc( DISP, XX-> gdrawable, XX-> gc, ELLIPSE_RECT, 0, 64*360);
+   XFLUSH;	  
    return true;
 }
 
@@ -787,6 +797,7 @@ apc_gp_fill_chord( Handle self, int x, int y, int dX, int dY, double angleStart,
       }
    }
    FILL_ANTIDEFECT_CLOSE;
+   XFLUSH;	  
    return true;
 }
 
@@ -796,6 +807,9 @@ apc_gp_fill_ellipse( Handle self, int x, int y,  int dX, int dY)
    DEFXX;
    int mix = 0;
 
+   if ( dX == 1 || dY == 1 ) /* Xorg bug */
+   	return apc_gp_bar( self, x - dX / 2, y - dY / 2, x + dX / 2, y + dY / 2);
+
    if ( PObject( self)-> options. optInDrawInfo) return false;
    if ( !XF_IN_PAINT(XX)) return false;
    if ( dX <= 0 || dY <= 0) return false;
@@ -810,6 +824,7 @@ apc_gp_fill_ellipse( Handle self, int x, int y,  int dX, int dY)
          XDrawArc( DISP, XX-> gdrawable, XX-> gc, x - ( dX + 1) / 2 + 1, y - dY / 2, dX-1, dY-1, 0, 64*360);
    }
    FILL_ANTIDEFECT_CLOSE;
+   XFLUSH;	  
    return true;
 }
 
@@ -847,6 +862,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point *points)
       warn( "Prima::Drawable::fill_poly: too many points");
    FILL_ANTIDEFECT_CLOSE;
    free( p);
+   XFLUSH;	  
    return true;
 }
 
@@ -883,6 +899,7 @@ apc_gp_fill_sector( Handle self, int x, int y, int dX, int dY, double angleStart
       }
    }
    FILL_ANTIDEFECT_CLOSE;
+   XFLUSH;	  
    return true;
 }
 
@@ -1134,6 +1151,7 @@ apc_gp_flood_fill( Handle self, int x, int y, Color color, Bool singleBorder)
       if ( s. lists[x]) 
          plist_destroy( s.lists[x]);
    free( s. lists);
+   XFLUSH;	  
 
    return ret;
 }
@@ -1329,6 +1347,7 @@ apc_gp_line( Handle self, int x1, int y1, int x2, int y2)
       gcv. line_width = 0;
       XChangeGC( DISP, XX-> gc, GCLineWidth, &gcv);
    }
+   XFLUSH;	  
    return true;
 }
 
@@ -1351,6 +1370,7 @@ apc_gp_rectangle( Handle self, int x1, int y1, int x2, int y2)
    }
    XDrawRectangle( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y2), x2 - x1, y2 - y1);
    XCHECKPOINT;
+   XFLUSH;	  
    return true;
 }
 
@@ -1385,6 +1405,7 @@ apc_gp_sector( Handle self, int x, int y,  int dX, int dY, double angleStart, do
        x, y,
        x + cos( angleEnd / GRAD) * dX / 2, y - sin( angleEnd / GRAD) * dY / 2
    );
+   XFLUSH;	  
    return true;
 }
 
@@ -1532,6 +1553,7 @@ apc_gp_set_pixel( Handle self, int x, int y, Color color)
    XSetForeground( DISP, XX-> gc, prima_allocate_color( self, color, nil));
    XDrawPoint( DISP, XX-> gdrawable, XX-> gc, x, REVERT( y));
    XX-> flags. brush_fore = 0;
+   XFLUSH;	  
    return true;
 }
 
@@ -1559,7 +1581,7 @@ static Point *
 gp_get_text_box( Handle self, const char * text, int len, Bool wide);
 
 static Bool
-gp_text_out_rotated( Handle self, const char * text, int x, int y, int len, Bool wide) 
+gp_text_out_rotated( Handle self, const char * text, int x, int y, int len, Bool wide, Bool * ok_to_not_rotate) 
 {
    DEFXX;
    int i;
@@ -1570,7 +1592,7 @@ gp_text_out_rotated( Handle self, const char * text, int x, int y, int len, Bool
    int psx, psy, dsx, dsy;
    Fixed rx, ry;
 
-   if ( !prima_update_rotated_fonts( XX-> font, text, len, wide, PDrawable( self)-> font. direction, &r)) 
+   if ( !prima_update_rotated_fonts( XX-> font, text, len, wide, PDrawable( self)-> font. direction, &r, ok_to_not_rotate)) 
       return false;
 
    for ( i = 0; i < len; i++) {
@@ -1731,6 +1753,7 @@ gp_text_out_rotated( Handle self, const char * text, int x, int y, int len, Bool
       if ( lw != XX-> font-> underlineThickness) 
          apc_gp_set_line_width( self, lw);
    }   
+   XFLUSH;	  
    return true;
 }   
 
@@ -1781,9 +1804,12 @@ apc_gp_text_out( Handle self, const char * text, int x, int y, int len, Bool utf
    RANGE2(x,y);
 
    if ( PDrawable( self)-> font. direction != 0) {
-      Bool ret = gp_text_out_rotated( self, text, x, y, len, utf8);
-      if ( utf8) free(( char *) text);
-      return ret;
+      Bool ok_to_not_rotate = false;
+      Bool ret = gp_text_out_rotated( self, text, x, y, len, utf8, &ok_to_not_rotate);
+      if ( !ok_to_not_rotate) {
+         if ( utf8) free(( char *) text);
+         return ret;
+      }
    }
 
    if ( !XX-> flags. paint_base_line)
@@ -1821,6 +1847,7 @@ apc_gp_text_out( Handle self, const char * text, int x, int y, int len, Bool utf
    }   
 
    if ( utf8) free(( char *) text);
+   XFLUSH;	  
    
    return true;
 }
@@ -2184,13 +2211,13 @@ gp_get_text_box( Handle self, const char * text, int len, Bool wide)
 
    if ( !XX-> flags. paint_base_line) {
       int i;
-      for ( i = 0; i < 5; i++) pt[i]. y += XX-> font-> font. descent;
+      for ( i = 0; i < 4; i++) pt[i]. y += XX-> font-> font. descent;
    }   
    
    if ( PDrawable( self)-> font. direction != 0) {
       int i;
-      double s = sin( PDrawable( self)-> font. direction / 572.9577951);
-      double c = cos( PDrawable( self)-> font. direction / 572.9577951);
+      double s = sin( PDrawable( self)-> font. direction / 57.29577951);
+      double c = cos( PDrawable( self)-> font. direction / 57.29577951);
       for ( i = 0; i < 5; i++) {
          double x = pt[i]. x * c - pt[i]. y * s;
          double y = pt[i]. x * s + pt[i]. y * c;
